@@ -20,7 +20,31 @@ class ManagerUtilsQuerySet(QuerySet):
         """
         return {obj.id: obj for obj in self}
 
-    def bulk_upsert(self, model_objs, unique_fields, update_fields):
+    def _get_upserts(self, model_objs_updated, model_objs_created, unique_fields):
+        """
+        Given a list of model objects that were updated and model objects that were created,
+        return the list of all model objects upserted. Doing this requires fetching all of
+        the models created with bulk create (since django can't return bulk_create pks)
+        """
+        # The upserted models default to the objects that were updated
+        upserted_models = model_objs_updated
+        # Fetch the objects that were created based on the uniqueness constraint. Note - only support the case
+        # where there is one update field so that we can perform an in query. TODO perform an OR query to gather
+        # the created values when there is more than one update field
+        if len(unique_fields) == 1:
+            unique_field = unique_fields[0]
+            upserted_models.extend(
+                self.filter(**{'{0}__in'.format(unique_field): (
+                    getattr(model_obj, unique_field) for model_obj in model_objs_created
+                )})
+            )
+        else:
+            raise NotImplementedError(
+                'bulk_upsert currently doesnt support returning upserts with more than one update field')
+
+        return upserted_models
+
+    def bulk_upsert(self, model_objs, unique_fields, update_fields, return_upserts=False):
         """
         Performs a bulk update or insert on a queryset.
         """
@@ -52,6 +76,10 @@ class ManagerUtilsQuerySet(QuerySet):
         self.model.objects.bulk_update(model_objs_to_update, update_fields)
         self.model.objects.bulk_create(model_objs_to_create)
 
+        # Optionally return the bulk upserted values
+        if return_upserts:
+            return self._get_upserts(model_objs_to_update, model_objs_to_create, unique_fields)
+
     def get_or_none(self, **query_params):
         """
         Get an object or return None if it doesn't exist.
@@ -105,7 +133,7 @@ class ManagerUtilsMixin(object):
         """
         return self.get_queryset().id_dict()
 
-    def bulk_upsert(self, model_objs, unique_fields, update_fields):
+    def bulk_upsert(self, model_objs, unique_fields, update_fields, return_upserts=False):
         """
         Performs a bulk update or insert on a list of model objects. Matches all objects in the queryset
         with the objs provided using the field values in unique_fields.
@@ -118,6 +146,8 @@ class ManagerUtilsMixin(object):
                 from the queryset.
             update_fields: A list of fields used from the objects in objs as fields when updating existing
                 models.
+            return_upserts: A flag specifying whether to return the upserted values. If True, this performs
+                an additional query to fetch any bulk created values.
 
         Signals: Emits a post_bulk_operation when a bulk_update or a bulk_create occurs.
 
@@ -173,7 +203,7 @@ class ManagerUtilsMixin(object):
             print TestModel.objects.count()
             7
         """
-        return self.get_queryset().bulk_upsert(model_objs, unique_fields, update_fields)
+        return self.get_queryset().bulk_upsert(model_objs, unique_fields, update_fields, return_upserts=return_upserts)
 
     def bulk_create(self, model_objs, batch_size=None):
         """
